def main():
    """ Takes in list of users and performs below:

    Enables VM, 
    Adds VM to email if required along with email address, 
    Transfer on zero (if required) and destination, 
    VM common password, 
    Auto-login

    :param x:
    :param y:
    :param z:

    :return r: 
    """
    return  