from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as file:
    long_description = file.read()

setup(
    name="odins_spear",
    version="1.5.0",
    url="https://github.com/Jordan-Prescott/odins_spear",
    author="Jordan Prescott",
    author_email="jordan@minimalmind.io",
    description="Python library for Odin API to build and manage Broadworks installations.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.10",
    install_requires=[
        "backports.tarfile==1.2.0",
        "certifi==2023.7.22",
        "cffi==1.16.0",
        "charset-normalizer==3.2.0",
        "colorama==0.4.6",
        "cryptography==41.0.7",
        "docutils==0.21.2",
        "et_xmlfile==2.0.0",
        "graphviz==0.20.1",
        "idna==3.4",
        "importlib_metadata==8.3.0",
        "jaraco.classes==3.4.0",
        "jaraco.context==6.0.1",
        "jaraco.functools==4.0.2",
        "keyring==25.3.0",
        "markdown-it-py==3.0.0",
        "mdurl==0.1.2",
        "more-itertools==10.4.0",
        "nh3==0.2.18",
        "numpy==1.26.4",
        "openpyxl==3.1.5",
        "pandas==2.2.2",
        "pkginfo==1.10.0",
        "pycparser==2.21",
        "Pygments==2.18.0",
        "PyJWT==2.8.0",
        "python-dateutil==2.9.0.post0",
        "pytz==2024.1",
        "pywin32-ctypes==0.2.3",
        "ratelimit==2.2.1",
        "readme_renderer==44.0",
        "requests==2.31.0",
        "requests-toolbelt==1.0.0",
        "rfc3986==2.0.0",
        "rich==13.7.1",
        "six==1.16.0",
        "tqdm==4.66.1",
        "twine==5.1.1",
        "tzdata==2024.1",
        "urllib3==2.0.4",
        "win32-setctime==1.1.0",
        "zipp==3.20.0",
    ],
)
